/***************************************************************************
                         mymap.cpp  -  the mymap shell
                             -------------------                                         
    begin                : 23 Nov 99
    copyright            : (C) 1999 by Jost Schenck
    email                : jost@schenck.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the ARTISTIC LICENSE.                           * 
 *                                                                         *
 ***************************************************************************/

#include <qdir.h>
#include <qfileinfo.h>
#include <qmessagebox.h>
#include <qprinter.h>
#include <qpainter.h>

#include <kaction.h>
#include <klocale.h>
#include <kiconloader.h>
#include <kfiledialog.h>
#include <kmenubar.h>
#include <khelpmenu.h>
#include <kio/netaccess.h>
// #include <kaboutdialog.h>

#include "mymap.h"
#include "mymap_doc.h"
#include "mymap_view.h"
#include "mymap_canvas.h"
#include "mymap_graphnode.h"
#include "mymap_global.h"
#include "docpropertiesdlg.h"

MyMap::MyMap() : KTMainWindow()
{
	resize( 700, 500 ); // FIXME
	pActColl = new KActionCollection( this );
//	setCaption(kapp->makeStdCaption( i18n("NoName") ) );
	printer = new QPrinter();
	printer->setFullPage( true ); // FIXME! I think true is better; make margins configurable
	printer->setOrientation( QPrinter::Landscape ); // FIXME! default configurable
	
	// create Document
	initNewDoc();
		
	// create canvas widget
	pView = new MyMapView( pDoc, this );
	setView( pView );

	initActions();
	initMenuBar();
	initToolBars();
	initStatusBar();
	
	changeCaption();
}

MyMap::~MyMap()
{
}

void MyMap::initNewDoc()
{
	pDoc = new MyMapDoc();
	connect( pDoc, SIGNAL( modifiedChanged() ), this, SLOT( changeCaption() ) );
}

void MyMap::initActions()
{
	debug("MyMap::initActions");
	acFileNew = new KAction( i18n("&New"), BarIcon("filenew"), Key_N+CTRL, this, SLOT( fileNew() ), pActColl, "filenew");
	acFileQuit = new KAction( i18n("&Quit"), Key_X+CTRL, this, SLOT( close() ), pActColl, "quit");
	acFileOpen = new KAction( i18n("&Open ..."), BarIcon("fileopen"), Key_O+CTRL, this, SLOT( open() ), pActColl, "open");
	acFileSave = new KAction( i18n("&Save"), BarIcon("filefloppy"), Key_S+CTRL, this, SLOT( save() ), pActColl, "save");
	acFileSaveAs = new KAction( i18n("Save &As ..."), 0, this, SLOT( saveAs() ), pActColl, "saveas");
	acFileSaveCopyAs = new KAction(i18n("Save &Copy As ..."),0,this,SLOT(saveCopyAs()),pActColl,"savecopyas");
	acFilePrint = new KAction( i18n("&Print"), BarIcon("fileprint"), Key_P+CTRL, this, SLOT( print() ), pActColl, "print");
	acFilePrintOptions=new KAction(i18n("&Edit Print Options ..."),0,this,SLOT( editPrinterOptions() ), pActColl, "editprintopts");
	acFileProperties = new KAction( i18n("&Document Properties ..."),0, this, SLOT( editDocProps() ), pActColl, "docprops");
}

void MyMap::initMenuBar()
{
	debug("MyMap::initMenuBar");
	QPopupMenu* m = new QPopupMenu( this );
		action("filenew")->plug( m );
		action("open")->plug( m );
		action("save")->plug( m );
		action("saveas")->plug( m );
		action("savecopyas")->plug( m );
		m->insertSeparator();
		action("print")->plug( m );
		action("editprintopts")->plug( m );
		m->insertSeparator();
		action("docprops")->plug( m );
		m->insertSeparator();
		action("quit")->plug( m );
	menuBar()->insertItem( i18n("&File"), m );
	m = new QPopupMenu( this );
//		pView->action("undo")->plug( m );
//		pView->action("redo")->plug( m );
//		m->insertSeparator();
		pView->action("cut")->plug( m );
		pView->action("copy")->plug( m );
//		pView->action("clone")->plug( m );
//		m->insertSeparator();
		pView->action("paste")->plug( m );
//		pView->action("pasteclone")->plug( m );
	menuBar()->insertItem( i18n("&Edit"), m );
	m = new QPopupMenu( this );
		pView->action("addmapsdlg")->plug( m );
		pView->action("deletemapsdlg")->plug( m );
		m->insertSeparator();
		pView->action("editmapprops")->plug( m );
	menuBar()->insertItem( i18n("&Maps"), m );
	m = new QPopupMenu( this );
		pView->action("editnodeprops")->plug( m );
		pView->action("editconprops")->plug( m );
		m->insertSeparator();
		pView->action("selectstyle")->plug( m );
	menuBar()->insertItem( i18n("&Node"), m );
	
	menuBar()->insertSeparator();
	
	m = helpMenu( i18n("(C) 1999 Jost Schenck\n"
		"jost@schenck.de\t\tuzsnty@uni-bonn.de\n\n"
		"MyMap - a mind mapping(tm) program\n"
		"MyMap's homepage is at: http://familie.schenck.de/jost/mymap\n\n"
		"MyMap comes with ABSOLUTELY NO WARRANTY.\n"
		"This is free software, and you are welcome to redistribute it\n"
		"under the conditions of the Artistic License.\n\n"
		"Mind mapping is a trademark of Tony Buzan.") );
	menuBar()->insertItem( i18n("&Help"), m );
}

void MyMap::initToolBars()
{
	debug("MyMap::initToolBars");
	action("filenew")->plug( toolBar(0) );
	action("save")->plug( toolBar(0));
	action("open")->plug( toolBar(0));
	action("print")->plug(toolBar(0));
	toolBar(0)->insertSeparator();
	pView->action("cut")->plug( toolBar(0) );
	pView->action("copy")->plug( toolBar(0));
	pView->action("paste")->plug(toolBar(0));
	
//	pView->action("newchild")->plug( toolBar(1) );
//	pView->action("delete")->plug( toolBar(1) );
//	pView->action("selectstyle")->plug( toolBar(1) );
//	acNewChild->plug( toolBar(1) );
//	acDeleteSelection->plug( toolBar(1) );
//	acNodeStyle->plug( toolBar(1) );
//	toolBar(1)->show();
}

void MyMap::changeCaption()
{
	if ( pDoc )
	{
		if ( pDoc->fileName() == "" )
			setCaption( kapp->makeStdCaption( i18n("Unknown"), pDoc->modified() ) );
		else
			setCaption( kapp->makeStdCaption( pDoc->fileName(), pDoc->modified() ) );
	}
	else
		setCaption( kapp->makeStdCaption("") );
}

void MyMap::initStatusBar()
{
	statusBar()->message(i18n("Welcome to MyMap!"), 5000);
}

void MyMap::cut()
{
}

void MyMap::copy()
{
}

void MyMap::paste()
{
}

void MyMap::fileNew()
{
	if ( queryClose() ) // FIXME?
		delete pDoc;
		
	pDoc = 0;
	initNewDoc();
	pView->setDoc( pDoc );		
}

bool MyMap::queryClose()
{
	// FIXME: session Management! How to do it when several views are open?
	if ( pDoc->modified() )
		switch ( QMessageBox::warning( this, "MyMap",
	                               i18n("This document has been modified.\nDo you want to save your work before exiting?"),
	                               i18n("&Save"),
	                               i18n("Do&n't Save"),
	                               i18n("&Cancel"),
	                               0, 2) )
		{
			case 0: // yes
 	       // save document here. If saving fails, return FALSE;
 	       save();
 	       return true;
			case 1: // no
 	       return true;
			default: // cancel
 	       return false;
 	 }
 	else
 		return true;
}


void MyMap::open()
{
//  QString fileToOpen=KFileDialog::getOpenFileName(QDir::homeDirPat(), "", this, i18n("Open File..."));
	debug("MyMap::open 1");
	KURL url = KFileDialog::getOpenURL( QString::null, "*", this );
	debug("MyMap::open 2");
	if( url.isEmpty() ) return;
	debug("MyMap::open 3");
  openFile( url );
  debug("MyMap::open 4");
}

void MyMap::openFile( const KURL& url )
{
	QString tmpfile;
	KIO::NetAccess::download( url, tmpfile );
	if( !tmpfile.isEmpty() && pDoc->load( tmpfile ) )
	{
		QFileInfo inf( tmpfile );
		pDoc->setFileName( url.filename() );
		pDoc->setURL( url );
		changeCaption();
		// FIXME: addRecentFile(fileToOpen);
	}
}

void MyMap::save()
{
	if ( pDoc->fileName() != "" )
		pDoc->save( pDoc->URL().filename() );
	else
		saveAs();
}

void MyMap::saveAs()
{
	KURL url = KFileDialog::getSaveURL(QDir::currentDirPath(), "", this, i18n("Save As..."));
	QString newName = url.filename();  // KFileDialog::getSaveFileName(QDir::currentDirPath(), "", this, i18n("Save As..."));
	if(!newName.isEmpty() && pDoc->save(newName) )
	{
		QFileInfo inf( newName );
		pDoc->setFileName( inf.fileName() );
		pDoc->setURL( url );
		changeCaption();
		// FIXME: addRecentFile(fileToOpen);
	}
}

void MyMap::saveCopyAs()
{
}

void MyMap::print()
{
	if ( pDoc && printer->setup( this ) )
	{
		QPainter p( printer );
		pDoc->print( printer, &p );
	}
}

void MyMap::editDocProps()
{
	DocPropertiesDlg* dlg = new DocPropertiesDlg( 0 , pDoc );
	dlg->showPage( 0 );
	dlg->show();
	debug("DocPropertiesDlg shown");
}

void MyMap::editPrinterOptions()
{

}

KAction* MyMap::action( const char* name )
{
	return pActColl->action(name);
}
	
KActionCollection* MyMap::actionCollection()
{
	return pActColl;
}

void MyMap::centerView()
{
	pView->centerView();
}

void MyMap::saveProperties( KConfig* sc )
{
	// TODO: session management! und zwar richtig tfte!
	if( !pDoc || ( pDoc->fileName() == "" && !pDoc->modified() ) )
  return;

  sc->writeEntry( "filename", pDoc->fileName() );
  sc->writeEntry( "modified", pDoc->modified() );

  if(pDoc->modified())
  {
    QString tmplocation = kapp->tempSaveName( name() );
    pDoc->save( tmplocation );
  }
}

void MyMap::readProperties( KConfig* sc )
{
	// TODO: session management! und zwar richtig tfte!
	QString filename = sc->readEntry("filename", "");
	int modified = sc->readNumEntry("modified", 0);

	if(!filename.isEmpty() && modified)
	{
		bool ok;
		QString fn = kapp->checkRecoverFile(filename, ok);
		if(ok)
		{
			pDoc->load( fn );
			pDoc->setFileName( filename );
			pDoc->setModified( true );
			changeCaption();
		}
	}
	else
	{
		if(!filename.isEmpty())
		{
			pDoc->load( filename );
			pDoc->setFileName( filename );
			pDoc->setModified( false );
      changeCaption();
		}
	}
}

/*void MyMap::fileQuit()
{
	if ( queryClose() )
		close();
}*/

