/***************************************************************************
                     mymap_canvas.h  -  displays mindmaps
                             -------------------                                         
    begin                : Tue Nov 23 1999                                           
    copyright            : (C) 1999 by Jost Schenck                         
    email                : jost@schenck.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the ARTISTIC LICENSE.                           * 
 *                                                                         *
 ***************************************************************************/


#ifndef MYMAP_CANVAS_H
#define MYMAP_CANVAS_H

#include <qwidget.h>

#include "mymap_global.h"

// forward declarations

class MyMapMap;
class MyMapSelection;
class MyMapGraphNode;
class MyMapItem;
class QPopupMenu;
class QLineEdit;
class KAction;
class KSelectAction;

/**MyMapCanvas draws a mindmap and allows
  *its manipulation.
  *@author Jost Schenck
  */

class MyMapCanvas : public QWidget
{
	Q_OBJECT
public: 
	MyMapCanvas(QWidget *parent, const char *name=0);
	~MyMapCanvas();
	
	MyMapMap* map() { return pMap; }
	void setMap( MyMapMap* m );
	MyMapGraphNode* appendNewChildAt( QCOORD x_, QCOORD y_ );

signals:
	void change();
	void checkTitles();
	
public slots:
	void updateView();
	void newDocLoaded();
	void appendNewChild();
	void deleteSelection();
	void mapPropertiesChanged();
	void changeNodeStyle( const QString& );
	void editCurrentProperties();
	void editCurrentConProperties();
	
protected:
	virtual void paintEvent( QPaintEvent* );	
	virtual void mouseMoveEvent( QMouseEvent* );
	virtual void mousePressEvent( QMouseEvent* );
	virtual void mouseReleaseEvent( QMouseEvent* );
	virtual void mouseDoubleClickEvent( QMouseEvent* );

protected slots:
	void editMode( bool = true );
	void editorTextChanged( const QString& );
	void slotAccel( int );
	
private:
	void initActions();
	void setCurrent( MyMapGraphNode* );
	void setEditorGeometry();
	
	MyMapMap* pMap;
	MyMapSelection* pSelection;
	// TODO: make this MyMapItem?
	MyMapGraphNode* pCurrent;
	
	QLineEdit* pEditor;
	bool bEditMode;
	MyMapGraphNode* pEditNode;
	
	bool bDragging; // true when mouse button was clicked on a node
	bool bMoved;    // true when mouse has MOVED with button clicked
	
	bool bResizing; // dito
	ResizeHandle rh;

	QCOORD xDragOffset, yDragOffset;
	QPopupMenu* pCanvasPopup, *pNodePopup;
	KAction *acNewChild, *acDeleteSelection, *acEditCurrentProperties;
	KSelectAction *acNodeStyle;
};

#endif
