/***************************************************************************
                          mymap_connection.cpp  -  description                              
                             -------------------                                         
    begin                : Sun Nov 28 1999                                           
    copyright            : (C) 1999 by Jost Schenck                         
    email                : jost@schenck.de                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the ARTISTIC LICENSE.                           * 
 *                                                                         *
 ***************************************************************************/


#include <qpointarray.h>

#include "mmglobal.h"
#include "mymap_connection.h"

MyMapConnection::MyMapConnection( /*MyMapGraphNode* n*/)
{
	// FIXME: maybe get this from n
	_linePen.setWidth( 1 );
	_style = mmcSIMPLELINE;
	_linePen.setStyle( Qt::SolidLine );
	_linePen.setColor( Qt::black );
	mmg = new MMGlobal();
}

MyMapConnection::~MyMapConnection()
{
}

KMMCONNECTORSTYLE MyMapConnection::style() { return _style; }
int MyMapConnection::lineWidth() { return _linePen.width(); }
const QColor& MyMapConnection::lineColor() { return _linePen.color(); }
const Qt::PenStyle MyMapConnection::lineStyle() { return _linePen.style(); }
const QPen& MyMapConnection::linePen() { return _linePen; }


void MyMapConnection::setStyle( KMMCONNECTORSTYLE sty ) { _style = sty; }
void MyMapConnection::setLineWidth( int t ) { _linePen.setWidth( t ); }
void MyMapConnection::setLineColor( const QColor& c ) { _linePen.setColor( c ); }
void MyMapConnection::setLineStyle( const Qt::PenStyle sty ) { _linePen.setStyle( sty ); }
void MyMapConnection::setLinePen( const QPen& p ) {	_linePen = p; }

void MyMapConnection::paint( int pX, int pY, int cX, int cY, QPainter* pPainter )
{
	pPainter->setPen( _linePen );
	
	if ( _style == mmcSIMPLELINE )
		pPainter->drawLine( pX, pY, cX, cY );

	if ( _style == mmcBEZIERLINE )
	{
		// FIXME: int w = _linePen.width(); _linePen.setWidth(1);
		// and then use w to draw a bezier-polygon
		QPointArray pa(4);
		pa.setPoint(3, pX, pY);
		pa.setPoint(0, cX, cY);
		pa.setPoint(1, pX + ((cX - pX)/2), cY);
		pa.setPoint(2, cX - ((cX - pX)/2) , pY);
		pPainter->drawQuadBezier(pa);
	}
}

QDomElement MyMapConnection::save( QDomDocument& d )
{
	QDomElement con = d.createElement( "connection" );
	switch ( _style )
	{
		case mmcSIMPLELINE:	con.setAttribute( "style", "simpleline" );
		case mmcBEZIERLINE: con.setAttribute( "style", "bezierline" );
	}
//	con.appendChild( d.createElement( "linepen", _linePen ) );
	con.appendChild( mmg->createElement( "linepen", _linePen, d ) );
	return con;
}

void MyMapConnection::load( QDomElement& e )
{
	if ( e.attribute("style") == "simpleline" )
		_style = mmcSIMPLELINE;
	else if ( e.attribute("style") == "bezierline" )
		_style = mmcBEZIERLINE;
		
	QDomNode n = e.firstChild();
	QDomElement element;
	while( !n.isNull() )
	{
		element = n.toElement();
		if ( element.tagName() == "linepen" )
//			_linePen = element.toPen();
			_linePen = mmg->toPen( element );
		n = n.nextSibling();
	}
}


