/***************************************************************************
               mymap_doc.cpp  -  holds info on mindmap documents
                             -------------------                                         
    begin                : Tue Nov 23 1999                                           
    copyright            : (C) 1999 by Jost Schenck                         
    email                : jost@schenck.de                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the ARTISTIC LICENSE.                           * 
 *                                                                         *
 ***************************************************************************/

#include <qbuffer.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qdom.h>
#include <qmsgbox.h>
#include <qpaintdevicemetrics.h>
#include <qpainter.h>
#include <qprinter.h>

#include <klocale.h>

#include "mymap_view.h"
#include "mymap_doc.h"
#include "mymap_graphnode.h"
#include "mymap_map.h"
#include "mymap_global.h"
#include "mymap.h"

MyMapDoc::MyMapDoc()
{
	_authorName = "John Doe";
	_authorEmail = "john@doe.com";
	MyMapMap* m = new MyMapMap(this);
	appendMap( m );
	setModified(false);
	
//	_pathName = "";
	_fileName = "";
}

MyMapDoc::~MyMapDoc()
{
}

bool MyMapDoc::save( const QString& sf )
{
	QDomDocument doc("mindmap");
  doc.appendChild( doc.createProcessingInstruction( "xml", "version=\"1.0\" encoding=\"UTF-8\"" ) );
	QDomElement mindmap = doc.createElement("mindmap");
	mindmap.setAttribute( "author", _authorName );
  mindmap.setAttribute( "email", _authorEmail );
  mindmap.setAttribute( "editor", "MyMap" );
  mindmap.setAttribute( "mime", "application/x-mymap" );
  doc.appendChild( mindmap );
  QDomElement map;
	// FIXME: allow multiple maps per file (-> tabbar like in a spreadsheet program)
	QListIterator<MyMapMap> it(lMaps);
	for (; it.current(); ++it)
		mindmap.appendChild( it.current()->save(doc) );
		
  QFile f( sf );
  f.open( IO_WriteOnly );
  QTextStream str( &f );
  str << doc;
  f.close();

	setModified( false );
  return true;
}

bool MyMapDoc::load( const QString& lf )
{
	// First recursively delete all maps and their nodes ...
	MyMapMap* m = lMaps.take();
	while ( m )
	{
		debug("delete map!");
		delete m;
		debug("map deleted!");
		m = lMaps.take();
	}
	debug("all maps deleted!");
	emit docCleared();
	
	// ... preparing ...
	debug("... preparing ...");
//	QFile f( lf );
//	f.open( IO_ReadOnly );
//	QTextStream str( &f );
//	QDomDocument doc( &f );
	QDomDocument doc( lf );
//	f.close();
	
	// ... check header and stuff ...
	debug("... check header and stuff ...");
	if ( doc.doctype().name() != "mindmap" )
		return false;
	QDomElement mindmap = doc.documentElement();
	if ( mindmap.attribute( "mime" ) != "application/x-mymap" )
		return false;
	_authorName = mindmap.attribute( "author" );
	_authorEmail= mindmap.attribute( "email" );

  // ... loop through all available maps ...
  debug("... loop throug all available maps ...");
//  bool bFoundMap = false; // so we can read old files where there's no <map> tag
  MyMapMap* pTmpMap = 0;
  QDomElement map = mindmap.namedItem("map").toElement();
  while ( !map.isNull() )
  {
//		bFoundMap = true;
		pTmpMap = new MyMapMap( this );
		pTmpMap->load( map );
		appendMap( pTmpMap );
		map = map.nextSibling().toElement();
  }
//  if ( !bFoundMap )
//		map = mindmap; // file is from MyMap 0.1.0, which didn't have the <map> tag
	
	// ... done.
	debug("... done.");
	setModified( false );
	emit newDocLoaded();	
	emit updateSignal();
	debug("end loading!");
	return true;
}

// const QString& MyMapDoc::pathName() { return _pathName; }
const KURL& MyMapDoc::URL() { return _url; }
const QString& MyMapDoc::fileName() { return _fileName; }
const QString& MyMapDoc::authorName() { return _authorName; }
const QString& MyMapDoc::authorEmail(){ return _authorEmail; }

// void MyMapDoc::setPathName( const QString& pn ) { _pathName = pn; }
void MyMapDoc::setURL( const KURL& url ) { _url = url; }
void MyMapDoc::setFileName( const QString& fn ) { _fileName = fn; }
void MyMapDoc::setAuthorName( const QString& an ) { _authorName = an; }
void MyMapDoc::setAuthorEmail( const QString& ae ) { _authorEmail = ae; }

void MyMapDoc::docChanged()
{
	emit updateSignal();
	setModified( true );
}

bool MyMapDoc::modified()
{
	return bModified;
}

void MyMapDoc::setModified( bool m )
{
	if ( m != bModified )
	{
		emit modifiedChanged();
		bModified = m;
	}
}

void MyMapDoc::insertMap( uint id, MyMapMap* m )
{
	lMaps.insert( id, m );
	emit docPropertiesChanged();
	// do some connects
}

bool MyMapDoc::removeMap( MyMapMap* m )
{
	bool r;
	if ( m )
	{
		// do some disconnects
		r = lMaps.remove(m);
		emit docPropertiesChanged();
		setModified( true );
		return r;
	}
		else return false;
}

void MyMapDoc::print( QPrinter* printer, QPainter* painter )
{
	QPaintDeviceMetrics m(printer);
	QRect geom;
	QListIterator<MyMapMap> it(lMaps);
	for(; it.current(); ++it)
	{
		// should I check for rootNode?
		geom = it.current()->rootNode()->geometry();
		it.current()->rootNode()->getMapGeometry( geom );
		// geom.moveTopLeft( QPoint(0,0) );
		if ( geom.width()>m.width() || geom.height()>m.height() )
		{
			// FIXME! keep proportions!
			painter->scale( m.width() / geom.width() , m.height() / geom.height() );
			painter->translate( printer->margins().width() - geom.left(),
			             printer->margins().height() - geom.top() );
		} else
		{
			painter->setViewport( ((m.width() - geom.width())/2)-geom.x(),
			                      ((m.height()-geom.height())/2)-geom.y(),
			                        m.width(), m.height() );
		}
		// painter->drawRect(it.current()->rootNode()->geometry());
		// painter->drawRect(geom);
		it.current()->rootNode()->paint( painter, true );
		printer->newPage(); // FIXME: creates a blank page on the last iteration
	}
}


