/***************************************************************************
                 mymap_doc.h  -  holds info on mindmap documents
                             -------------------                                         
    begin                : Tue Nov 23 1999                                           
    copyright            : (C) 1999 by Jost Schenck                         
    email                : jost@schenck.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the ARTISTIC LICENSE.                           * 
 *                                                                         *
 ***************************************************************************/


#ifndef MYMAP_DOC_H
#define MYMAP_DOC_H

#include <iostream.h>

#include <qobject.h>
#include <qcolor.h>
#include <qlist.h>

#include <kurl.h>

// forward declarations

class QPrinter;
class QPainter;
class QDomDocument;
class MyMapGraphNode;
class MyMapMap;
class MyMapView;

/**MyMapDoc holds information about a mindmap
  *document.
  *@author Jost Schenck
  */

class MyMapDoc : public QObject
{
	friend MyMapView;
	Q_OBJECT
public:
	MyMapDoc();
	~MyMapDoc();
	bool save( const QString& );
	bool load( const QString& );
	void print( QPrinter* printer, QPainter* painter );
	bool modified();
//	const QString& pathName();
	const KURL& URL();
	const QString& fileName();
	const QString& authorName();
	const QString& authorEmail();
	
//	void setPathName( const QString& );
	void setURL( const KURL& );
	void setFileName( const QString& );
	void setAuthorName( const QString& );
	void setAuthorEmail( const QString& );
	void setModified( bool );
	
	QList<MyMapMap>& maps() { return lMaps; }
	
	QCOORD forkVSpace() { return _forkVSpace; }
	QCOORD forkHSpace() { return _forkHSpace; }
	void setForkVSpace ( QCOORD v ) { _forkVSpace = v; }
	void setForkHSpace ( QCOORD h ) { _forkHSpace = h; }
	
//	virtual void paintContent( QPainter& painter, const QRect& rect, bool transparent = FALSE );
//	virtual bool initDoc();

signals:
	void docCleared();
	void updateSignal();
	void newDocLoaded();
	void modifiedChanged();
	void bgColorChanged(); // FIXME! This is rubbish!
	void docPropertiesChanged();

public slots:
	void docChanged();

private:
	void insertMap( uint, MyMapMap* );
	void appendMap( MyMapMap* m ) { insertMap( lMaps.count(), m ); setModified( true ); }
	bool removeMap( MyMapMap* );

	MyMapGraphNode* pRootNode;
//	QString _pathName;
	KURL _url;
	QString _fileName;
	bool bModified;
	
	QList<MyMapMap> lMaps;
	
	QCOORD _forkVSpace;
	QCOORD _forkHSpace;
	
	QString _authorName, _authorEmail;
};

#endif
