/***************************************************************************
          mymap_graphnode.cpp  -  graphical representation of a node
                             -------------------                                         
    begin                : Wed Nov 24 1999                                           
    copyright            : (C) 1999 by Jost Schenck                         
    email                : jost@schenck.de                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the ARTISTIC LICENSE.                           * 
 *                                                                         *
 ***************************************************************************/

#include "mmglobal.h" // includes qdom.h
#include "mymap_graphnode.h"
#include "mymap_connection.h"

/*
	TODO: rewrite all the methods that deal with _style, _standardChildNodeStyle,
	bForceChildStyle, bFreeMove etc. THIS IS RUBBISH.
*/


MyMapGraphNode::MyMapGraphNode( MyMapGraphNode* p ) : MyMapItem() // don't pass p to Item ctor to assure MyMapGraphNode::appendChild is called
{
	debug("Node C'tor");
	// most of this shouldn't be hardcoded like this
	_width = 70;
	_height = 40;
	_style = mmELLIPSE;
	_standardChildNodeStyle = mmELLIPSE;
	bForceChildStyle = false;
	bFreeMove = true;
	bAlignChildren = false;
	_text = ""; // "Hallo Welt!";
	_linepen.setColor( Qt::black );
	_linepen.setWidth( 1 );
	_linepen.setStyle( Qt::SolidLine );
	_textpen.setColor( Qt::black );
	_textpen.setStyle( Qt::SolidLine );
	_textpen.setWidth( 1 );
	_backColor = Qt::cyan;
	_isRightSide = 1;
	setConOffsets();
	autoGeometry();
	mmg = new MMGlobal();
	// create connection object
	pConnection = new MyMapConnection();
	debug("Node C'tor end");
	if( p )
		p->appendChild( this );
	else
		pParent = 0;
}

MyMapGraphNode::~MyMapGraphNode()
{
	if ( pParent )
		pParent->removeChild( this );
		
	// TODO: maybe introduce some sort of reference counting when the cloning feature
	//       has been implemented. Depends on the way I implement this.
	QListIterator<MyMapItem> it(lChildren);
	for (; it.current(); ++it)
		delete it.current();
}

// QCOORD MyMapGraphNode::width() { return _width; }
// QCOORD MyMapGraphNode::height() { return _height; }
const QString& MyMapGraphNode::text() { return _text; }
KMMNODESTYLE MyMapGraphNode::style() { return _style; }
KMMNODESTYLE MyMapGraphNode::standardChildNodeStyle() { return _standardChildNodeStyle; }
int MyMapGraphNode::lineWidth()   { return _linepen.width(); }
const QColor& MyMapGraphNode::lineColor() { return _linepen.color(); }
const Qt::PenStyle MyMapGraphNode::lineStyle(){ return _linepen.style(); }
const QPen& MyMapGraphNode::linePen()     { return _linepen; }
const QColor& MyMapGraphNode::textColor() { return _textpen.color(); }
const QColor& MyMapGraphNode::backColor() { return _backColor; }
const QFont&  MyMapGraphNode::font()      { return _font; }
bool MyMapGraphNode::isCopyConnStyle()    { return bCopyConnStyle; }
// void MyMapGraphNode::setWidth ( int w ) { _width = w; }
// void MyMapGraphNode::setHeight( int h ) { _height = h; }
void MyMapGraphNode::setText( const QString& t )
{
	_text = t;
	// FIXME: make this smarter
	// if ( _style == mmFORKSTYLE )
		autoGeometry();
}

void MyMapGraphNode::autoGeometry()
{
	// FIXME: make this smarter
	QFontMetrics m( _font );
	setWidth( m.width( _text ) + 4 );
	setConOffsets();
	if ( isAlignChildren() )
		autoPosChildren( false );
	if ( _style != mmFORKSTYLE )
		setWidth( (_width<_height)?_height:_width );
}

void MyMapGraphNode::setStyle( KMMNODESTYLE sty )
{
	QListIterator<MyMapItem> it(lChildren);
	MyMapGraphNode* pTmpNode;
	// if style changes back from FORK to other, childs have to be movable again
	if ( (_style == mmFORKSTYLE) && (sty != mmFORKSTYLE) )
	{
		setAlignChildren( false );
		setForceChildStyle( false );
		for (;it.current();++it)
			if ( it.current()->rtti() == GraphNode )
				it.current()->setFreeMove( true );
	}
	if (sty == mmFORKSTYLE )
	{
		_height = 20; // FIXME!		// has to be done before call to setConOffsets;
		pConnection->setStyle( mmcBEZIERLINE );
		setAlignChildren( true );
		setForceChildStyle( true );
		for (;it.current();++it)
			if ( it.current()->rtti() == GraphNode )
			{
				pTmpNode = dynamic_cast<MyMapGraphNode*>( it.current() );
				pTmpNode->setStyle( mmFORKSTYLE );
				pTmpNode->setFreeMove( false );
			}
	}
	else
	{
		pConnection->setStyle( mmcSIMPLELINE );
		setForceChildStyle( false );
	}
		
	_style = sty;
	setConOffsets();

	// The following tries to realize smart choosing of a style for new children.
	// FIXME: I should get rid of the above testing for Forkstyle and just use two
	// attributes: standardChildNodeStyle and forceChildNodeStyle. The latter would be
	// true for all fork childs. FIX before all this gets all too complicated :)
	if ( parent() && ( parent()->style() != mmFORKSTYLE ) )
		parent()->setStandardChildNodeStyle( _style );
		
	setStandardChildNodeStyle( sty );
	if ( isAlignChildren() )
		autoPosChildren( false );
}

void MyMapGraphNode::setStandardChildNodeStyle( KMMNODESTYLE sty )
{ _standardChildNodeStyle = sty; }

void MyMapGraphNode::setLineWidth( int lt ) { _linepen.setWidth( lt ); }
void MyMapGraphNode::setLineColor( const QColor& c ) { _linepen.setColor( c ); }
void MyMapGraphNode::setLineStyle( const Qt::PenStyle s ) { _linepen.setStyle( s ); }
void MyMapGraphNode::setLinePen( const QPen& p ) { _linepen = p; }
void MyMapGraphNode::setTextColor( const QColor& c ) { _textpen.setColor( c ); }
void MyMapGraphNode::setBackColor( const QColor& c ) { _backColor = c; }
void MyMapGraphNode::setFont( const QFont& f ) { _font = f; }
void MyMapGraphNode::setCopyConnStyle( bool c ) { bCopyConnStyle = c; }
	
// QCOORD MyMapGraphNode::x() { return xPos; }
// QCOORD MyMapGraphNode::y() { return yPos; }
int MyMapGraphNode::parentConX() { return xPos + _parentConXOffset; }
int MyMapGraphNode::parentConY() { return yPos + _parentConYOffset; }
int MyMapGraphNode::childConX()  { return xPos + _childConXOffset; }
int MyMapGraphNode::childConY()  { return yPos + _childConYOffset; }

void MyMapGraphNode::setConOffsets()
{
	switch ( _style )
	{
	case mmTRANSPARENT:
	case mmFORKSTYLE:
			if ( _isRightSide == 1 )
			{
				_parentConXOffset = -4;
				_childConXOffset  = _width + 4;
			}
			else
			{
				_parentConXOffset = _width + 4;
				_childConXOffset  = -4;
			}
			_parentConYOffset = _height;
			_childConYOffset  = _height;
		break;
	case mmELLIPSE:
	case mmRECTANGLE:
	case mmROUNDRECT:
	default:
		_parentConXOffset = _width  / 2;
		_parentConYOffset = _height / 2;
		_childConXOffset  = _width  / 2;
		_childConYOffset  = _height / 2;
	}
}

QRect MyMapGraphNode::editorBounds()
{
	/* FIXME */
	return QRect( xPos, yPos, _width, _height );
}

/* QRect MyMapGraphNode::geometry()
{
	return QRect( xPos, yPos, _width, _height );
}  */

bool MyMapGraphNode::isAlignChildren() { return bAlignChildren; }
int  MyMapGraphNode::isRightSide() { return _isRightSide; }
bool MyMapGraphNode::isForceChildStyle() { return bForceChildStyle; }
// bool MyMapGraphNode::freeMove() { return bFreeMove; }

void MyMapGraphNode::moveTo( QCOORD _x, QCOORD _y )
{
	xPos = _x;
	yPos = _y;
	setRightSideAuto();
	if ( _style != mmFORKSTYLE )
	{
		QListIterator<MyMapItem> it(lChildren);
		for (; it.current(); ++it)
			if ( it.current()->rtti() == GraphNode )
				dynamic_cast<MyMapGraphNode*>(it.current())->setRightSideAuto();
	}
}

// void MyMapGraphNode::moveTo( QPoint pos ) { moveTo( pos.x(), pos.y() ); }
void MyMapGraphNode::setAlignChildren( bool b ) { bAlignChildren = b; }
void MyMapGraphNode::setRightSide( int i ) { _isRightSide = i; }

void MyMapGraphNode::setRightSideAuto()
{ /* TODO: Abhngig von Position zur Mutter */
	if ( parent() )
	{
    if ( parent()->style() != mmFORKSTYLE )
			if ( ( parent()->x() + (parent()->width() / 2) ) >
			     ( xPos + ( _width / 2 ) ) )
				setRightSide( -1 );
			else
				setRightSide( 1 );
		else
			setRightSide( parent()->isRightSide() );
	}
	setConOffsets();
	if ( isAlignChildren() ) // ( _style == mmFORKSTYLE )
		autoPosChildren( false );
}

void MyMapGraphNode::setForceChildStyle( bool b )
{
	bForceChildStyle = b;
	if ( bForceChildStyle )
	{
		QListIterator<MyMapItem> it(lChildren);
		for (; it.current(); ++it)
			if ( it.current()->rtti() == GraphNode )
				dynamic_cast<MyMapGraphNode*>(it.current())->setStyle( _style );
	}
}

// void MyMapGraphNode::setFreeMove( bool b ) { bFreeMove = b; }

MyMapGraphNode* MyMapGraphNode::parent() { return dynamic_cast<MyMapGraphNode*>(pParent); }
// MyMapGraphNode* MyMapGraphNode::childAt( int i ) { return lChildren.at(i); /* TODO */ }
// int MyMapGraphNode::childPos( MyMapGraphNode* n ) { return lChildren.find(n); /* TODO */ }

/* MyMapGraphNode* MyMapGraphNode::itemAt( const QPoint& p )
{
	if ( QRect(xPos, yPos, _width, _height ).contains( p ) )
		return this;
	else
	{
		QListIterator<MyMapGraphNode> it(lChildren);
		MyMapGraphNode* node = 0;
		for (; it.current(); ++it )
		{
			node = it.current()->itemAt( p );
			if ( node ) return node;
		}
		return 0;
	}
}*/
	
void MyMapGraphNode::setParent( MyMapGraphNode* p ) { pParent = p; }

void MyMapGraphNode::appendChild( MyMapGraphNode* _c )
{
  if ( _c )
  {
    _c->setParent( this );
    lChildren.append( _c );
//    MyMapConnector* pTmpConn = new MyMapConnector();
		if ( _style == mmFORKSTYLE )
		{
			_c->setStyle( mmFORKSTYLE );
			_c->setFreeMove( false );
		}
		else
			_c->setFreeMove( true );
		if ( isAlignChildren() )
			autoPosChildren( true );
	}
}

bool MyMapGraphNode::removeChild( MyMapGraphNode* _c )
{
	if ( _c )
	{
		_c->setParent( 0 );
		bool r = lChildren.remove(_c);
		if ( isAlignChildren() )
			autoPosChildren( true );
		return r;
	}
	else
		return false;
}

/* void MyMapGraphNode::reparent( MyMapGraphNode* p )
{
	// where and how should I check for circular reparenting? Is this right here?
	if ( p && !isSuccessor( p ) )
	{
		if ( pParent )
			pParent->removeChild( this );
		p->appendChild( this );
	}
}*/

/* bool MyMapGraphNode::isSuccessor( MyMapGraphNode* n )
{
	if ( n == this ) return true;
	QListIterator<MyMapGraphNode> it(lChildren);
	for (; it.current(); ++it)
		if ( it.current()->isSuccessor(n) ) return true;
	return false;
} */

void MyMapGraphNode::paint( QPainter* pPainter, bool recursive, bool select, bool current )
{
	// first draw all our children!
	if ( lChildren.count() )
	{
		QListIterator<MyMapItem> it(lChildren);
		for (; it.current(); ++it)
			it.current()->paint( pPainter, recursive, select, current );
	}
	
	// now draw connection to parent

	if ( parent() && pConnection )
		pConnection->paint( parent()->childConX(),
		                    parent()->childConY(),
		                    parentConX(),
		                    parentConY(), pPainter);

	QRect entryRect ( xPos, yPos, _width, _height);
	pPainter->setFont( _font );
	pPainter->setBrush( _backColor );
	pPainter->setPen( _linepen );

	switch ( _style )
	{
	case mmELLIPSE:
		pPainter->drawEllipse( entryRect );
		pPainter->setPen( _textpen );
		pPainter->drawText ( entryRect, Qt::AlignCenter|Qt::WordBreak, _text );
	break;
	case mmRECTANGLE:
		pPainter->drawRect( entryRect );
		pPainter->setPen( _textpen );
		pPainter->drawText ( entryRect, Qt::AlignCenter, _text );
		break;
	case mmROUNDRECT:
		pPainter->drawRoundRect( entryRect, 10, 10); // evtl. Rundung relativ zur Gre
		pPainter->setPen( _textpen );
		pPainter->drawText ( entryRect, Qt::AlignCenter, _text );
		break;
	case mmTRANSPARENT:
		pPainter->setPen( _textpen );
		pPainter->drawText ( entryRect, Qt::AlignCenter, _text );
		break;
  case mmFORKSTYLE:
		pPainter->drawLine ( entryRect.x() - 4, entryRect.y() + entryRect.height(),
		                     entryRect.x() + entryRect.width() + 4,
		                     entryRect.y() + entryRect.height() );
		pPainter->setPen   ( _textpen );
		pPainter->drawText ( entryRect, Qt::AlignCenter, _text );
		break;

	default:
		pPainter->drawText ( entryRect, Qt::AlignCenter, "FEHLER");
  }
}

QDomElement MyMapGraphNode::save( QDomDocument& doc )
{
	QDomElement node = doc.createElement("node");
	node.setAttribute("text", _text);
	node.setAttribute("x", xPos);
	node.setAttribute("y", yPos);
	node.setAttribute("width", _width);
	node.setAttribute("height", _height);
	switch ( _style )
	{
		case mmELLIPSE: node.setAttribute("style", "ellipse"); break;
		case mmRECTANGLE: node.setAttribute("style", "rectangle"); break;
		case mmROUNDRECT: node.setAttribute("style", "roundrect"); break;
		case mmTRANSPARENT: node.setAttribute("style", "transparent"); break;
		case mmFORKSTYLE: node.setAttribute("style", "forkstyle"); break;
	}
//	node.appendChild( doc.createElement( "font", _font ) );
//	node.appendChild( doc.createElement( "linepen", _linepen ) );
//	node.appendChild( doc.createElement( "textpen", _textpen ) );
  node.appendChild( mmg->createElement( "linepen", _linepen, doc ) );
	node.appendChild( mmg->createElement( "textpen", _textpen, doc ) );
	if ( pConnection )
		node.appendChild( pConnection->save( doc ) );
	
	if ( lChildren.count() )
	{
		QListIterator<MyMapItem> it(lChildren);
		for (; it.current(); ++it)
			node.appendChild( it.current()->save( doc ) );
	}
	
	return node;
}

void MyMapGraphNode::load( QDomElement& e )
{
	_text   = e.attribute( "text" );
	xPos    = e.attribute( "x" ).toUInt();
	yPos    = e.attribute( "y" ).toUInt();
	_width  = e.attribute( "width" ).toUInt();
	_height = e.attribute( "height" ).toUInt();
	QString sty = e.attribute( "style" );
	if ( sty == "ellipse")  setStyle(mmELLIPSE);
	else
	if ( sty == "rectangle" )  setStyle(mmRECTANGLE);
	else
	if ( sty == "roundrect" )  setStyle(mmROUNDRECT);
	else
	if ( sty == "forkstyle" )  setStyle(mmFORKSTYLE);
	else
	if ( sty == "transparent" )  setStyle(mmTRANSPARENT);
	else
		setStyle(mmELLIPSE);
		
	setRightSideAuto();
	
	MyMapGraphNode* tmpNode;
	QDomNode n = e.firstChild();
	QDomElement element;
	while( !n.isNull() )
	{
		element = n.toElement();
		if ( !element.isNull() )
		{
//			if ( element.tagName() == "font" )
//				_font = element.toFont();
			if ( element.tagName() == "linepen" )
				_linepen = mmg->toPen( element );
			if ( element.tagName() == "textpen" )
				_textpen = mmg->toPen( element );
			if ( element.tagName() == "node" )
			{
				tmpNode = new MyMapGraphNode( this );
				tmpNode->load( element );
			}
			if ( element.tagName() == "connection" )
			{
				if ( !pConnection )
					pConnection = new MyMapConnection();
				pConnection->load( element );
			}
		}
		n = n.nextSibling();
	}
}

QCOORD MyMapGraphNode::treeHeight()
{
	QCOORD th = 0;
	if ( lChildren.count() )
	{
		// TODO: take non-GraphNode items into account
		QListIterator<MyMapItem> it(lChildren);
		for (; it.current(); ++it)
			if ( it.current()->rtti() == GraphNode )
				th += dynamic_cast<MyMapGraphNode*>(it.current())->treeHeight();
	}
	if ( th < ( _height + 15 /*pDoc->forkVSpace()*/ ) )
		th = _height + 15 /*pDoc->forkVSpace()*/;
	return th;
}

void MyMapGraphNode::autoPosChildren( bool climbup )
{
	// FIXME: should I check here for my own isAlignChildren?
	if ( climbup && parent() && parent()->isAlignChildren() )
		parent()->autoPosChildren( true );
	else
	{
		QCOORD cy = (yPos + (_height/2)) - (treeHeight()/2);
		QCOORD th, cx;
		MyMapGraphNode* pTmpNode;
		QListIterator<MyMapItem> it(lChildren);
		// TODO: take non-GraphNode items into account
		for (; it.current(); ++it)
			if ( it.current()->rtti() == GraphNode )
			{
				debug("CHILD ----------------------------------");
				pTmpNode = dynamic_cast<MyMapGraphNode*>(it.current());
				th = (pTmpNode->treeHeight()/2);// - (pTmpNode->height()/2);
				cy += th;
				cx = xPos + _childConXOffset + ( _isRightSide * 35 ); // use configurable HSpace
				if ( _isRightSide == -1 )
					cx -= pTmpNode->width();
				pTmpNode->moveTo( cx, cy );
				cy += th;
				pTmpNode->autoPosChildren();
			}
	}
}

void MyMapGraphNode::getMapGeometry( QRect& r )
{
	if ( xPos < r.x() ) r.setX( xPos );
	if ( yPos < r.y() ) r.setY( yPos );
	if ( xPos+_width > r.right() ) r.setRight( xPos+_width );
	if ( yPos+_height > r.bottom() ) r.setBottom( yPos+_height );

	QListIterator<MyMapItem> it(lChildren);
	for (; it.current(); ++it)
		if ( it.current()->rtti() == GraphNode )
			dynamic_cast<MyMapGraphNode*>(it.current())->getMapGeometry( r );
}

bool MyMapGraphNode::isResizable()
{
	if ( _style == mmFORKSTYLE )
		return false;
	else
		return true;
}

