/***************************************************************************
           mymap_graphnode.h  -  graphical representation of a node
                             -------------------                                         
    begin                : Wed Nov 24 1999                                           
    copyright            : (C) 1999 by Jost Schenck                         
    email                : jost@schenck.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the ARTISTIC LICENSE.                           * 
 *                                                                         *
 ***************************************************************************/


#ifndef MYMAP_GRAPHNODE_H
#define MYMAP_GRAPHNODE_H

#include <qdom.h>
#include <qlist.h>
#include <qstring.h>
#include <qcolor.h>
#include <qpainter.h>
#include <qpen.h>

#include "mymap_global.h"
#include "mymap_item.h"

// forward declarations

class MyMapConnection;
class MMGlobal;

/**Graphical representation of a mindmap node,
  *used for drawing the whole mindmap
  *@author Jost Schenck
  */

class MyMapGraphNode : public MyMapItem
{
public:
	MyMapGraphNode( MyMapGraphNode* p = 0 );
	~MyMapGraphNode();
	
	ItemType rtti() { return GraphNode; }
	
//	int width();
//	int height();
	const QString& text();
	KMMNODESTYLE style();
	KMMNODESTYLE standardChildNodeStyle();
	int lineWidth();
	const QColor& lineColor();
	const Qt::PenStyle lineStyle();	
	const QPen& linePen();
	const QColor& textColor();
	const QColor& backColor();
	const QFont&  font();
	bool isCopyConnStyle();
	bool isResizable();
	
//	void setWidth( int w );
//	void setHeight( int h );
  void setText( const QString& t );
	void setStyle( KMMNODESTYLE );
	void setStandardChildNodeStyle( KMMNODESTYLE );
	void setLineWidth( int );
	void setLineColor( const QColor& );
	void setLineStyle( const Qt::PenStyle );
	void setLinePen( const QPen& );
	void setTextColor( const QColor& );
	void setBackColor( const QColor& );
	void setFont( const QFont& );
	void setCopyConnStyle( bool );
	
//	QCOORD x();
//	QCOORD y();
	int  parentConX();
	int  parentConY();	
	int  childConX();
	int  childConY();
	QRect editorBounds();
//	QRect geometry();
	
	bool isAlignChildren();
//	bool isSelected();
//	bool isCurrent();
	bool isForceChildStyle();
	int  isRightSide();
///	bool freeMove();
	bool isSelectable() { return true; }
	bool isCurrentable() { return true; }
	bool belongsToParent() { return false; }
	bool canBeParent() { return true; }
	
//	const QList<MyMapGraphNode>& children() { return lChildren; }
	
	QCOORD treeHeight();
	void autoPosChildren( bool = false );
	
	void moveTo( QCOORD _x, QCOORD _y );
//	void moveTo( QPoint pos );
	void setAlignChildren( bool );
	void setRightSide( int );
	void setRightSideAuto();
	void setForceChildStyle( bool );
//	void setFreeMove( bool );
	
	MyMapGraphNode* parent(); // should return 0 if there is a parent, but it's no GraphNode (shouldn't happen anyway)
	MyMapConnection* parentConnection() { if ( pParent ) return pConnection; else return 0; }
//	MyMapGraphNode* childAt( int );
//	int childPos( MyMapGraphNode* );
//	MyMapGraphNode* itemAt( const QPoint& p );
	
	void setParent( MyMapGraphNode* p );
  // make this ( MyMapItem* ) ? Or is this correct overloading?
	void appendChild( MyMapGraphNode* );
	bool removeChild( MyMapGraphNode* );
//	void reparent( MyMapGraphNode* );

	void paint( QPainter* pPainter, bool recursive = true, bool select = false, bool current = false );
	
	void getMapGeometry( QRect& );
	QDomElement save( QDomDocument& );
	void load( QDomElement& e);
//	bool isSuccessor( MyMapGraphNode* );

private:
	void autoGeometry();
	void setConOffsets();

//	MyMapGraphNode* pParent;
	MyMapConnection* pConnection;
//  QList<MyMapGraphNode> lChildren;

  QString _text;

	bool bAlignChildren;
	bool bFreeMove;
	bool bCopyConnStyle;
	bool bForceChildStyle;
	int  _isRightSide;
	
//	QCOORD xPos;
//	QCOORD yPos;
	QCOORD _parentConXOffset;
	QCOORD _parentConYOffset;
	QCOORD _childConXOffset;
	QCOORD _childConYOffset;
//	QCOORD _width, _height;
	
	KMMNODESTYLE _style;
	KMMNODESTYLE _standardChildNodeStyle;
	QColor _backColor; // replace with QBrush
	QFont _font;
	QPen _linepen;
	QPen _textpen;
	MMGlobal* mmg;
};

#endif
