/***************************************************************************
                   mymap_item.cpp  -  an item in a mind map
                             -------------------                                         
    begin                : Sat Apr 8 2000                                           
    copyright            : (C) 2000 by Jost Schenck                         
    email                : jost@schenck.de                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the ARTISTIC LICENSE.                           * 
 *                                                                         *
 ***************************************************************************/


#include "mymap_item.h"

MyMapItem::MyMapItem( MyMapItem* p )
{
	debug("Item C'tor");
	if( p )
		p->appendChild( this );
	else
		pParent = 0;
	debug("Item C'tor end");
}

MyMapItem::~MyMapItem()
{
}

int MyMapItem::width() { return _width; }
int MyMapItem::height() { return _height; }
void MyMapItem::setWidth( int w ) { _width = w; }
void MyMapItem::setHeight( int h ) { _height = h; }

QCOORD MyMapItem::x() { return xPos; }
QCOORD MyMapItem::y() { return yPos; }
QRect MyMapItem::geometry() { return QRect( xPos, yPos, _width, _height ); }

const QList<MyMapItem>& MyMapItem::children() { return lChildren; }

void MyMapItem::moveTo( QCOORD _x, QCOORD _y ) { xPos = _x; yPos = _y; }
void MyMapItem::moveTo( QPoint pos ) { moveTo( pos.x(), pos.y() ); }
	
MyMapItem* MyMapItem::parent() { return pParent; }
MyMapItem* MyMapItem::childAt( int i ) { return lChildren.at(i); }

int MyMapItem::childPos( MyMapItem* n ) { return lChildren.find(n); }

MyMapItem* MyMapItem::itemAt( const QPoint& p )
{
	if ( QRect(xPos, yPos, _width, _height ).contains( p ) )
		return this;
	else
	{
		QListIterator<MyMapItem> it(lChildren);
		MyMapItem* item = 0;
		for (; it.current(); ++it )
		{
			item = it.current()->itemAt( p );
			if ( item ) return item;
		}
		return 0;
	}
}

bool MyMapItem::isSuccessor( MyMapItem* n )
{
	if ( n == this ) return true;
	QListIterator<MyMapItem> it(lChildren);
	for (; it.current(); ++it)
		if ( it.current()->isSuccessor(n) ) return true;
	return false;
}
	
void MyMapItem::setParent( MyMapItem* p ) { pParent = p; }

void MyMapItem::appendChild( MyMapItem* c )
{
  if ( c )
  {
    c->setParent( this );
    lChildren.append( c );
   }
}

bool MyMapItem::removeChild( MyMapItem* c )
{
	if ( c )
	{
		c->setParent( 0 );
		return lChildren.remove( c );
	}
	else
		return false;
}

void MyMapItem::reparent( MyMapItem* p )
{
	if ( p && !isSuccessor( p ) )
	{
		if ( pParent )
			pParent->removeChild( this );
		p->appendChild( this );
	}
}
	
bool MyMapItem::freeMove() { return bFreeMove; }

void MyMapItem::setFreeMove( bool bfm ) { bFreeMove = bfm; }

// void MyMapItem::getMapGeometry( QRect& );

void MyMapItem::paint( QPainter* pPainter, bool recursive, bool select, bool current )
{
	debug("------SCHEIIIIIISSSSSSSSEEEEEE-------------");
}
