/***************************************************************************
                    mymap_item.h  -  an item in a mind map
                             -------------------                                         
    begin                : Sat Apr 8 2000                                           
    copyright            : (C) 2000 by Jost Schenck                         
    email                : jost@schenck.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the ARTISTIC LICENSE.                           * 
 *                                                                         *
 ***************************************************************************/

#include <qrect.h>
#include <qpoint.h>
#include <qlist.h>
#include <qdom.h>

#include "mymap_global.h"

#ifndef MYMAP_ITEM_H
#define MYMAP_ITEM_H


/**MyMapItem is the base class for Nodes,
  *Checkboxes, Images etc.
  *@author Jost Schenck
  */

class MyMapItem
{
public: 
	MyMapItem( MyMapItem* p = 0 );
	virtual ~MyMapItem();
	
	virtual ItemType rtti() = 0;
	
	virtual int width();
	virtual int height();
	virtual void setWidth( int );
	virtual void setHeight( int );
	
	virtual QCOORD x();
	virtual QCOORD y();
	virtual QRect geometry();

	virtual bool isResizable() = 0;
	virtual bool isSelectable() = 0;
	virtual bool isCurrentable() = 0;
	virtual bool canBeParent() = 0;
	/** Returns true, if this item can be considered 'a part of its parent', i.e. checkboxes
	  * or maybe images. This affects the behaviour of the selection, reparenting etc.*/
	virtual bool belongsToParent() = 0;
	
//	bool isAlignChildren();
//	bool isForceChildStyle();
//	int  isRightSide();
	virtual bool freeMove(); // get rid of this
	
	virtual const QList<MyMapItem>& children();
	
//	QCOORD treeHeight();
//	void autoPosChildren( bool = false );
	
	virtual void moveTo( QCOORD _x, QCOORD _y );
	virtual void moveTo( QPoint pos );
//	void setAlignChildren( bool );
//	void setRightSide( int );
//	void setRightSideAuto();
//	void setForceChildStyle( bool );
	virtual void setFreeMove( bool ); // get rid of this
	
	virtual MyMapItem* parent();
	virtual MyMapItem* childAt( int );
	virtual int childPos( MyMapItem* );
	virtual MyMapItem* itemAt( const QPoint& p );
	virtual	bool isSuccessor( MyMapItem* );
	
	virtual void setParent( MyMapItem* p );
	virtual void appendChild( MyMapItem* );
	virtual bool removeChild( MyMapItem* );
	virtual void reparent( MyMapItem* );

	virtual void paint( QPainter* pPainter, bool recursive = true, bool select = false, bool current = false );
	
	virtual void getMapGeometry( QRect& ) = 0;
	virtual QDomElement save( QDomDocument& ) = 0;
	virtual void load( QDomElement& e) = 0;

protected:
	MyMapItem* pParent;
  QList<MyMapItem> lChildren;
	
  bool bFreeMove;

	QCOORD xPos;
	QCOORD yPos;
	QCOORD _width, _height;
	
};

#endif
