/***************************************************************************
               mymap_map.cpp  -  a map (page) in a mymap document
                             -------------------                                         
    begin                : Tue Dec 14 1999                                           
    copyright            : (C) 1999 by Jost Schenck                         
    email                : jost@schenck.de                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the ARTISTIC LICENSE.                           * 
 *                                                                         *
 ***************************************************************************/


#include <klocale.h>

#include "mymap_doc.h"
#include "mymap_map.h"
#include "mymap_graphnode.h"

MyMapMap::MyMapMap( MyMapDoc* doc )
{
	pDoc = doc;
	pRootNode = new MyMapGraphNode();
	pRootNode->moveTo(2000, 2000);
	pRootNode->setText( i18n("New Map") );
	pRootNode->setStandardChildNodeStyle( mmFORKSTYLE );
	_bgColor = Qt::white;
	
	if ( pDoc )
		connect( this, SIGNAL( mapPropertiesChanged() ), pDoc, SLOT( docChanged() ) );
}

MyMapMap::~MyMapMap()
{
	if ( pRootNode )
		delete pRootNode;
}

const QString MyMapMap::title()
{
	if ( pRootNode )
		return pRootNode->text();
	else
		return i18n("Unknown");
}

MyMapItem* MyMapMap::itemAt( const QPoint& p )
{
	if ( pRootNode )
		return pRootNode->itemAt( p );
	else
		return 0;
}

QDomElement MyMapMap::save( QDomDocument& doc )
{
	QDomElement map = doc.createElement("map");
	
	map.setAttribute( "bgcol", _bgColor.name() );

  if ( pRootNode )
  {
  	QDomElement e = pRootNode->save( doc );
  	map.appendChild( e );
	}
	return map;
}

void MyMapMap::load( QDomElement& e )
{
	pRootNode = new MyMapGraphNode();
	
	// set Attributes!
	
  QDomElement n = e.namedItem("node").toElement();
  if ( !n.isNull() )
  	pRootNode->load( n );
}

