/***************************************************************************
                   mymap_selection.cpp  -  selection object
                             -------------------                                         
    begin                : Mon Nov 29 1999                                           
    copyright            : (C) 1999 by Jost Schenck                         
    email                : jost@schenck.de                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the ARTISTIC LICENSE.                           * 
 *                                                                         *
 ***************************************************************************/


#include <qpainter.h>

#include "mymap_selection.h"
// #include "mymap_graphnode.h"
#include "mymap_item.h"

MyMapSelection::MyMapSelection()
{
	bResizable = true;
}

MyMapSelection::~MyMapSelection()
{
}

void MyMapSelection::addObject( MyMapItem* o )
{
	if ( o && o->isSelectable() && lSelection.find( o ) == -1 )
	{
		lSelection.append( o );
		if ( !o->isResizable() )
			bResizable = false;
		setGeometry();
	}
}

void MyMapSelection::removeObject( MyMapItem* o )
{
	if ( o && lSelection.find( o ) )
	{
		lSelection.remove( o );
		if ( !o->isResizable() )
		{
			bResizable = true;
			QListIterator<MyMapItem> it(lSelection);
			for (; it.current(); ++it)
				if ( !it.current()->isResizable() ) bResizable = false;
		}
		setGeometry();
	}
}

bool MyMapSelection::toggleObject( MyMapItem* o )
{
	if ( o && (lSelection.find( o ) == -1) )
	{
		if ( o->isSelectable() )
		{
			lSelection.append( o );
			setGeometry();
			return true;
		}
		else return false;
	}
	else
	{
		lSelection.remove( o );
		setGeometry();
		return false;
	}
}

bool MyMapSelection::isSelected( MyMapItem* o )
{
	if ( lSelection.find( o ) != -1 )
		return true;
	else
		return false;
}
	
void MyMapSelection::clear()
{
	lSelection.clear();
	bResizable = true;
}

const QList<MyMapItem>& MyMapSelection::selection()
{
	return lSelection;
}
	
void MyMapSelection::paint( QPainter* p )
{
	/* TODO */
	if ( lSelection.count() )
	{
		QPen pen;
		pen.setStyle( Qt::DotLine );
		pen.setColor( Qt::black );
		pen.setWidth( 1 );
		p->setPen( pen );
		p->setBrush( Qt::NoBrush );
		p->drawRect( xPos, yPos, w, h );
		
		pen.setStyle( Qt::SolidLine );
		pen.setColor( bResizable ? Qt::black : Qt::white );
		p->setPen( pen );
		p->setBrush( bResizable ? Qt::black : Qt::lightGray );

		p->drawRect( xPos - 4, yPos - 4, 5, 5 );
		p->drawRect( xPos + w - 1, yPos - 4, 5, 5 );
		p->drawRect( xPos + w - 1, yPos + h - 1, 5, 5 );
		p->drawRect( xPos - 4, yPos + h - 1, 5, 5 );
		p->drawRect( xPos + (w/2) - 2, yPos - 4, 5, 5 );
		p->drawRect( xPos + w - 1, yPos + (h/2) - 2, 5, 5 );
		p->drawRect( xPos + (w/2) - 2, yPos + h - 1, 5, 5 );
		p->drawRect( xPos - 4, yPos + (h/2) - 2, 5, 5 );

		pen.setStyle( Qt::SolidLine );
		p->setPen( pen );
		p->setBrush( Qt::NoBrush );
		QListIterator<MyMapItem> it(lSelection);
		for (; it.current(); ++it)
			p->drawRect( it.current()->x(), it.current()->y(), 4, 4 );
	}
}


ResizeHandle MyMapSelection::getHandle( const QPoint& p )
{
	if ( QRect( xPos - 4, yPos - 4, 5, 5 ).contains( p ) ) return rhNW;
	if ( QRect( xPos + (w/2) - 2, yPos - 4, 5, 5 ).contains( p ) ) return rhN;
	if ( QRect( xPos + w - 1, yPos - 4, 5, 5 ).contains( p ) ) return rhNE;
	if ( QRect( xPos + w - 1, yPos + (h/2) - 2, 5, 5 ).contains( p ) ) return rhE;
	if ( QRect( xPos + w - 1, yPos + h - 1, 5, 5 ).contains( p ) ) return rhSE;
	if ( QRect( xPos - 4, yPos + h - 1, 5, 5 ).contains( p ) ) return rhSW;
	if ( QRect( xPos + (w/2) - 2, yPos + h - 1, 5, 5 ).contains( p ) ) return rhS;
	if ( QRect( xPos - 4, yPos + (h/2) - 2, 5, 5 ).contains( p ) ) return rhW;
	return rhNone;		
}


void MyMapSelection::setGeometry()
{
	QCOORD sX = 0, sY = 0, hX = 0, hY = 0;
	
	QListIterator<MyMapItem> it(lSelection);
	for (; it.current(); ++it)
		{
			if ( it.current()->x() < sX || !sX )
				sX = it.current()->x();
			if ( it.current()->y() < sY || !sY )
				sY = it.current()->y();
			if ( ( it.current()->x() + it.current()->width() ) > hX || !hX )
				hX = it.current()->x() + it.current()->width();
			if ( ( it.current()->y() + it.current()->height() ) > hY || !hY )
				hY = it.current()->y() + it.current()->height();
		}

	xPos = sX; yPos = sY;
	w = hX - sX; h = hY - sY;
	debug("xPos: %i \nyPos: %i \nw: %i \nh: %i", xPos, yPos, w, h );
}

void MyMapSelection::move( QCOORD x_, QCOORD y_ )
{
	QListIterator<MyMapItem> it(lSelection);
	for (; it.current(); ++it)
		if ( it.current()->freeMove() )
			it.current()->moveTo( it.current()->x() + x_ - xPos,
			                      it.current()->y() + y_ - yPos );
	setGeometry();
}

void MyMapSelection::reparent( MyMapItem* p )
{
	// where do I check for circular reparenting?
	if ( p->canBeParent() )
	{
		QListIterator<MyMapItem> it(lSelection);
		for (; it.current(); ++it)
			it.current()->reparent( p );
	}
}

void MyMapSelection::resize( ResizeHandle rh, const QPoint& p )
{
	if ( rh != rhNone )
	{
		QCOORD newx, newy;
		// FIXME: I think this got a problem with signed/unsigned values
		float dw, dh;
		switch(rh)
		{
			case rhN: dw = 1; dh = ( yPos - p.y() + h ) / h;
			          newx = xPos; newy = p.y(); break;
			case rhS: dw = 1; dh = ( yPos - p.y() ) / h;
			          newx = xPos; newy = yPos; break;
			case rhW: dh = 1; dw = ( xPos - p.x() + w ) / w;
			          newx = p.x(); newy = yPos; break;
			case rhE: dh = 1; dw = ( p.x() - xPos ) / w;
			          newx = xPos; newy = yPos; break;
		}
		debug(" x: %i -> %i \n y: %i -> %i \n dw: %d \n dh: %d", xPos, newx, yPos, newy, dw, dh);
		QListIterator<MyMapItem> it(lSelection);
		for (; it.current(); ++it)
		{
			it.current()->moveTo( newx + (it.current()->x() - xPos)*dw,
			                      newy + (it.current()->y() - yPos)*dh );
			it.current()->setWidth( it.current()->width() * dw );
			it.current()->setHeight( it.current()->height() * dh );
		}
		setGeometry();
	}
}

bool MyMapSelection::isEmpty()
{
	return lSelection.isEmpty();
}


