/***************************************************************************
                          mymap_view.cpp  -  description                              
                             -------------------                                         
    begin                : Wed Nov 24 1999                                           
    copyright            : (C) 1999 by Jost Schenck                         
    email                : jost@schenck.de                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the ARTISTIC LICENSE.                           * 
 *                                                                         *
 ***************************************************************************/

#include "mymap_view.h"

// #include <qaction.h>
#include <qlist.h>
#include <qpushbutton.h>
#include <qscrollview.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qpixmap.h>

#include <klocale.h>
#include <kaction.h>
#include <kiconloader.h>

#include "mymap_doc.h"
#include "mymap_map.h"
#include "mymap_canvas.h"
#include "mymap_graphnode.h"
#include "mymapview_tabbar.h"
#include "mymapdlg_addmaps.h"
#include "mymapdlg_delmaps.h"

/* XPM */
static const char*center[]={
"12 12 2 1",
". c #000000",
"# c None",
".##########.",
"#.#.####.#.#",
"##..####..##",
"#...####...#",
"#####..#####",
"####....####",
"####....####",
"#####..#####",
"#...####...#",
"##..####..##",
"#.#.####.#.#",
".##########."};

MyMapView::MyMapView(MyMapDoc *doc, QWidget *parent, const char *name)
:	QWidget( parent, name )
//	: QSplitter( parent, name )
{
	pDoc = 0;
	pCanvas = 0;
	pActColl = new KActionCollection( this );
	iActiveMap = 0;
	
	QBoxLayout* layout = new QVBoxLayout( this );
	layout->setAutoAdd( true );
	
	pScrollView = new QScrollView( this );
	pTabBar = new MyMapViewTabBar( this );
	connect( pTabBar, SIGNAL( selected(int) ), this, SLOT( showMap(int) ) );
	connect( pTabBar, SIGNAL( deleteMap() ), this, SLOT( deleteMap() ) );
	connect( pTabBar, SIGNAL( addMap(int,bool)),this,SLOT( addMap(int, bool)));
	
	// FIXME
	pCanvas = new MyMapCanvas( pScrollView );
	pScrollView->addChild( pCanvas );
	pCanvas->resize(4000, 4000);
	connect( pCanvas, SIGNAL( checkTitles() ), pTabBar, SLOT( checkTitles()));
	
	setDoc( doc );
	
	// Corner Widget
	QPushButton* cw = new QPushButton( this );
	cw->setPixmap( QPixmap( center ) );
	QToolTip::add( cw, i18n("Center Root") );
	pScrollView->setCornerWidget( cw );
	connect( cw, SIGNAL( clicked() ), this, SLOT( centerView() ) );
	
	// get doc data
	initActions();
	docPropertiesChanged();
}

MyMapView::~MyMapView()
{
}

void MyMapView::setDoc( MyMapDoc* d )
{
	debug("setDoc");
	if ( pDoc )
	{
		disconnect( pDoc, SIGNAL( docCleared() ), this, SLOT( clearView() ) );
		disconnect( pDoc, SIGNAL( newDocLoaded() ), this, SLOT( centerView() ) );
		disconnect( pDoc, SIGNAL( newDocLoaded() ), this, SLOT( docPropertiesChanged() ) );
		disconnect( pDoc, SIGNAL( docPropertiesChanged() ), this, SLOT( docPropertiesChanged()));
	}
	pDoc = d;
	if ( pDoc )
	{
		connect( pDoc, SIGNAL( docCleared() ), this, SLOT( clearView() ) );
		connect( pDoc, SIGNAL( newDocLoaded() ), this, SLOT( centerView() ) );
		connect( pDoc, SIGNAL( newDocLoaded() ), this, SLOT( docPropertiesChanged() ) );
		connect( pDoc, SIGNAL( docPropertiesChanged() ), this, SLOT( docPropertiesChanged()));
	}
	debug("setDoc before pCanvas->setMap");
	if ( pCanvas )
		pCanvas->setMap( pDoc->maps().getFirst() );
	debug("setDoc ENDE");
}

void MyMapView::appendNewChild()
{
	pCanvas->appendNewChild();
}

void MyMapView::centerView()
{
	// When there are multiple canvas, FIXME
	if ( pCanvas->map() && pCanvas->map()->rootNode() )
		pScrollView->setContentsPos( pCanvas->map()->rootNode()->x() + (pCanvas->map()->rootNode()->width()/2) - (pScrollView->width()/2),
		                             pCanvas->map()->rootNode()->y() + (pCanvas->map()->rootNode()->height()/2)- (pScrollView->height()/2) );
}

void MyMapView::docPropertiesChanged()
{
	// get all maps
	QList<MyMapMap> lMaps = pDoc->maps();
	pTabBar->setupTabs( lMaps );
	showMap( iActiveMap );
}

void MyMapView::showMap( int id )
{
	iActiveMap = id;
	QList<MyMapMap> m = pDoc->maps();
	pCanvas->setMap( m.at(iActiveMap) );
}

void MyMapView::clearView()
{
	debug("clearView");
	// FIXME: delete all tabs
	QTab* ptab = 0; int i = 0;
	for(; pTabBar->tab(i); i++)
	{
		ptab = pTabBar->tab(i);
		pTabBar->removeTab( ptab );
		ptab = 0;
	}
	debug("clearView 1");
	if ( pCanvas )
		pCanvas->setMap(0);
	debug("clearView 2");
	pTabBar->repaint();
	debug("clearView ENDE");
}

void MyMapView::deleteMap( int m )
{
	uint id = (m == -1) ? iActiveMap : m;
	MyMapMap* map = pDoc->maps().at(id);
	if ( m == -1 && iActiveMap > 0 ) iActiveMap--;
	pDoc->removeMap( map );
	delete map;
	if ( m == -1 )
		pTabBar->setCurrentTab( iActiveMap );
}

void MyMapView::deleteMapsDialog()
{
	// TODO
	DelMapsDlg dlg;
	dlg.setView( this );
	dlg.show();
}

void MyMapView::addMap( int id, bool before )
{
	// FIXME
	iActiveMap = before?id:id+1;
	pDoc->insertMap( iActiveMap, new MyMapMap( pDoc ) );
	pTabBar->setCurrentTab( iActiveMap );
}

void MyMapView::addMapsDialog()
{
	// TODO
	AddMapsDlg dlg;
	dlg.setView( this );
	dlg.show();
}

void MyMapView::editMapProperties()
{
	// TODO
}

KAction* MyMapView::action( const char* name )
{
	return pActColl->action(name);
}
	
KActionCollection* MyMapView::actionCollection()
{
	return pActColl;
}

void MyMapView::initActions()
{
	debug("MyMapView::initActions");
	acEditCut = new KAction( i18n("C&ut"), QIconSet(BarIcon("editcut")), Key_X+CTRL, this, SLOT( cut() ), pActColl, "cut");
	acEditCopy = new KAction( i18n("&Copy"), QIconSet(BarIcon("editcopy")), Key_C+CTRL, this, SLOT( copy() ), pActColl, "copy");
	acEditPaste = new KAction( i18n("&Paste"), QIconSet(BarIcon("editpaste")), Key_V+CTRL, this, SLOT( paste() ), pActColl, "paste");
	acNewChild = new KAction( i18n("create &new child"), QIconSet(BarIcon("newnode")), 0, canvas(), SLOT( appendNewChild() ), pActColl, "newchild");
	acDeleteSelection = new KAction( i18n("&delete"), QIconSet(BarIcon("delnode")), 0, canvas(), SLOT( deleteSelection() ), pActColl, "delete");
	acDelMapsDialog = new KAction( i18n("&Delete Maps ..."), 0, this, SLOT(deleteMapsDialog()), pActColl, "deletemapsdlg");
	acAddMapsDialog = new KAction( i18n("&Add Maps ..."), 0, this, SLOT(addMapsDialog()), pActColl, "addmapsdlg" );
	acEditMapProperties = new KAction( i18n("&Edit Map Properties ..."), 0, this, SLOT(editMapProperties()), pActColl, "editmapprops");
	acEditNodeProperties = new KAction( i18n("Edit &Node ..."), 0, pCanvas, SLOT(editCurrentProperties()), pActColl, "editnodeprops");
	acEditConProperties = new KAction( i18n("Edit &Connection ..."), 0, pCanvas, SLOT(editCurrentConProperties()), pActColl, "editconprops");
	
	acNodeStyle = new KSelectAction( i18n("&Set Node Style"), 0, pActColl, "selectstyle" );
	connect( acNodeStyle, SIGNAL( activated( const QString& ) ), canvas(), SLOT(changeNodeStyle(const QString&)) );
	acNodeStyle->setEditable( false );
	QStringList items = QStringList() << i18n("Ellipse") << i18n("Fork") << i18n("Rectangle") << i18n("Rounded Rectangle") << i18n("Transparent");
	acNodeStyle->setItems( items );
}

void MyMapView::cut()
{
}

void MyMapView::copy()
{
}

void MyMapView::paste()
{
}



