/***************************************************************************
              mymapdlg_delmaps.cpp  -  dialog for deleting maps
                             -------------------                                         
    begin                : Sat Feb 5 2000                                           
    copyright            : (C) 2000 by Jost Schenck                         
    email                : jost@schenck.de                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the ARTISTIC LICENSE.                           * 
 *                                                                         *
 ***************************************************************************/

#include <qlayout.h>
#include <qlabel.h>
#include <qspinbox.h>

#include <klocale.h>

#include "mymap_map.h"
#include "mymap_view.h"
#include "mymap_doc.h"
#include "mymapdlg_delmaps.h"

DelMapsDlg::DelMapsDlg(QWidget *parent, const char *name )
  : KDialogBase(parent,name, true, i18n("delete maps"), Ok|Cancel )
{
	debug("AddMapsDlg ctor");
	// TODO: check tutorial on developer.kde.org if I did all this correctly
	QWidget* w = new QWidget( this );
	setMainWidget( w );
	QLabel *pLabel;
	
	QGridLayout* gl = new QGridLayout( w );
	gl->setMargin( marginHint() );
	gl->setSpacing( spacingHint() );
                                  		
		pLabel = new QLabel( w );
		pLabel->setText( i18n("Delete maps") );
		pLabel->resize( pLabel->sizeHint() );
		gl->addMultiCellWidget( pLabel, 0, 0, 0, 2 );
		gl->addRowSpacing( 0, pLabel->height() );
		
		pLabel = new QLabel( w );
		pLabel->setText( i18n("from:") );
		pLabel->resize( pLabel->sizeHint() );
		gl->addWidget( pLabel, 1, 0 );
		gl->addRowSpacing( 1, pLabel->height() );
		
		pFromSpin = new QSpinBox( w );
		gl->addWidget( pFromSpin, 1, 1 );
		gl->addRowSpacing( 1, pFromSpin->height() );
		
		pMapLabelFrom = new QLabel( w );
		pMapLabelFrom->setText("fromlabel");
		gl->addWidget( pMapLabelFrom, 1, 2 );
		
		pLabel = new QLabel( w );
		pLabel->setText( i18n("to:") );
		pLabel->resize( pLabel->sizeHint() );
		gl->addWidget( pLabel, 2, 0 );
		gl->addRowSpacing( 2, pLabel->height() );
		
		pToSpin = new QSpinBox( w );
		gl->addWidget( pToSpin, 2, 1 );
		gl->addRowSpacing( 2, pToSpin->height() );
		
		pMapLabelTo = new QLabel( w );
		pMapLabelTo->setText("tolabel");
		gl->addWidget( pMapLabelTo, 2, 2 );
		
		gl->setColStretch( 0, 0 );
		gl->setColStretch( 1, 0 );
		gl->setColStretch( 2, 1 );

		connect( pFromSpin, SIGNAL( valueChanged(int) ), this, SLOT( slotFromChanged() ) );
		connect( pToSpin, SIGNAL( valueChanged(int) ), this, SLOT( slotToChanged() ) );
	
	w->setMinimumSize( w->sizeHint() );
}

DelMapsDlg::~DelMapsDlg()
{
}

void DelMapsDlg::setView( MyMapView* v )
{
	pView = v;
	if ( pView )
	{
		lMaps = pView->doc()->maps();
		debug("setView - map: %i  num: %i", pView->activeMapID(), lMaps.count() );
		pFromSpin->setRange( 0, lMaps.count() - 1 );
		pToSpin->setRange( 0, lMaps.count() - 1 );
		pFromSpin->setValue( pView->activeMapID() );
		pToSpin->setValue( pView->activeMapID() );
		setLabels();
	}
}

void DelMapsDlg::setLabels()
{
	if ( pView )
	{
		// set Labels
		pMapLabelFrom->setText( "\"" + lMaps.at( pFromSpin->value() )->title() + "\"" );
		pMapLabelTo->setText( "\"" + lMaps.at( pToSpin->value() )->title() + "\"" );
		// set ranges
		// pFromSpin->setRange( 0, pToSpin->value() );
		// pToSpin->setRange( pFromSpin->value(), maxmap );
	}
}

void DelMapsDlg::slotOk()
{
	debug("slotOk");
	if ( pView )
	{
		int i;
		for(i = pFromSpin->value(); i <= pToSpin->value(); i++ )
			pView->deleteMap( pFromSpin->value() );
		pView->docPropertiesChanged();
		KDialogBase::slotOk();
	}
}

void DelMapsDlg::slotFromChanged()
{
	if ( pFromSpin->value() > pToSpin->value() )
		pToSpin->setValue( pFromSpin->value() );
	setLabels();
}

void DelMapsDlg::slotToChanged()
{
	if ( pToSpin->value() < pFromSpin->value() )
		pFromSpin->setValue( pToSpin->value() );
	setLabels();
}
