/***************************************************************************
                          mymapview_tabbar.cpp  -  description                              
                             -------------------                                         
    begin                : Tue Jan 4 2000                                           
    copyright            : (C) 2000 by Jost Schenck                         
    email                : jost@schenck.de                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the ARTISTIC LICENSE.                           * 
 *                                                                         *
 ***************************************************************************/

#include <qpopupmenu.h>

#include <klocale.h>

#include "mymapview_tabbar.h"
#include "mymap_map.h"


MyMapViewTabBar::MyMapViewTabBar(QWidget *parent, const char *name ) : QTabBar(parent,name)
{
	setShape( QTabBar::TriangularBelow );
	pOnTabMenu = new QPopupMenu( this );
	pOnTabMenu->insertItem( i18n("delete map"), 0 );
	pOnTabMenu->insertItem( i18n("insert map after"), 1 );
	pOnTabMenu->insertItem( i18n("insert map before"), 2 );
	connect( pOnTabMenu, SIGNAL( activated( int )), this, SLOT( mapMenuActivated( int ) ) );
	
	pOnEmptyMenu = new QPopupMenu( this );
	pOnEmptyMenu->insertItem( i18n("append map"), 0 );
	connect( pOnEmptyMenu, SIGNAL( activated( int )), this, SLOT( barMenuActivated( int ) ) );
}

MyMapViewTabBar::~MyMapViewTabBar()
{
}

/** Builds the TabBar from the list of maps in the document.
  */

void MyMapViewTabBar::setupTabs( QList<MyMapMap>& l )
{
	QTab* ptab = 0;
	uint i = 0;
	uint countmax = ( l.count() >= tabList()->count() ) ? l.count() : tabList()->count();
	for (; i < countmax ; i++)
	{
		ptab = tab(i);
		if ( !ptab )
		{
			ptab = new QTab;
			addTab( ptab );
			ptab->id = i;
		}
		if ( !l.at(i) )
			removeTab( ptab );
		else
			ptab->label = l.at(i)->title();
	}
	layoutTabs();
	repaint();
}

void MyMapViewTabBar::mousePressEvent( QMouseEvent* e )
{
	QTabBar::mousePressEvent( e );
	if ( e->button() == Qt::RightButton )
	{
		setCurrentTab( selectTab( e->pos() ) );
		clickedMapId = tabList()->find( selectTab( e->pos() ));
		if ( clickedMapId != -1 )
			pOnTabMenu->popup( mapToGlobal(e->pos()) );
		else
			pOnEmptyMenu->popup( mapToGlobal(e->pos()) );
	}
}

void MyMapViewTabBar::mapMenuActivated( int id )
{
	switch ( id )
	{
		case 0: emit deleteMap(); break;
		case 1: emit addMap( clickedMapId, false ); break;
		case 2: emit addMap( clickedMapId, true ); break;
	}
}

void MyMapViewTabBar::barMenuActivated( int id )
{
	if ( id == 0 )
		emit addMap( tabList()->count() - 1, false );
}

void MyMapViewTabBar::checkTitles()
{
//	QListIterator<QTab> it(tabList());
//	for (; it.current(); ++it)
//		it.current()->label = l.at(i)->title();
}