/***************************************************************************
                  propertiesdlg.cpp  -  edit item properties
                             -------------------                                         
    begin                : Mon Jul 19 1999                                           
    copyright            : (C) 1999 by Jost Schenck                         
    email                : jost@schenck.de                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; Artistic license applies.              *
 *                                                                         *
 ***************************************************************************/


#include <kfontdialog.h>
#include <klocale.h>

#include "propertiesdlg.h"
#include "mymap_global.h"
#include "mymap_graphnode.h"
#include "mymap_connection.h"


PropertiesDlg::PropertiesDlg(QWidget *parent, MyMapGraphNode* node, const char *name ) : QTabDialog(parent,name)
{
  pNode = node;
  // QTabDialog should find its right size automatically. But somehow this doesn't work
  // here. Doesn't matter, I want to get rid of those generated Dialogs anyhow, and do
  // some with proper layout management.
  resize(580, 350);
  genProps = new GeneralPropertiesDlg( this );
  connProps = new ConnectionPropertiesDlg( this );
  if ( pNode )
  {
    setupGenProps();
    if ( pNode->parentConnection() )
      setupConnProps();
  }
  setOKButton(i18n("OK"));
  setCancelButton(i18n("Cancel"));
  connect(this, SIGNAL( applyButtonPressed() ), this, SLOT( slotOKClicked() ) );
  connect( connProps->pCopyLineStyleCheckbox, SIGNAL( clicked() ), this, SLOT( slotChangeCopyFromEntry() ) );
  addTab( genProps, i18n("&General"));
  addTab( connProps, i18n("&Connection"));
}

PropertiesDlg::~PropertiesDlg()
{

}

void PropertiesDlg::setupGenProps()
{
   genProps->pItemStyleCombo->insertItem(i18n("Fork"), mmFORKSTYLE);
   genProps->pItemStyleCombo->insertItem(i18n("Ellipse"), mmELLIPSE);
   genProps->pItemStyleCombo->insertItem(i18n("Rectangle"), mmRECTANGLE);
   genProps->pItemStyleCombo->insertItem(i18n("Round Rectangle"), mmROUNDRECT);
   genProps->pItemStyleCombo->insertItem(i18n("Transparent"), mmTRANSPARENT);

   genProps->pLineStyleCombo->insertItem(i18n("No pen"), 0);
   genProps->pLineStyleCombo->insertItem(i18n("_________"), 1);
   genProps->pLineStyleCombo->insertItem(i18n("- - - - -"), 2);
   genProps->pLineStyleCombo->insertItem(i18n("* * * * *"), 3);
   genProps->pLineStyleCombo->insertItem(i18n("-*-*-*-*-"), 4);
   genProps->pLineStyleCombo->insertItem(i18n("-**-**-**"), 5);
   switch( pNode->lineStyle() )
   {
     case NoPen:  genProps->pLineStyleCombo->setCurrentItem(0); break;
     case SolidLine:  genProps->pLineStyleCombo->setCurrentItem(1); break;
     case DashLine:  genProps->pLineStyleCombo->setCurrentItem(2); break;
     case DotLine:  genProps->pLineStyleCombo->setCurrentItem(3); break;
     case DashDotLine:  genProps->pLineStyleCombo->setCurrentItem(4); break;
     case DashDotDotLine:  genProps->pLineStyleCombo->setCurrentItem(5); break;
   }

   genProps->pTextEdit->setText( pNode->text() );
   genProps->pItemStyleCombo->setCurrentItem( pNode->style() );
   genProps->pTextColorBtn->setColor( pNode->textColor() );
   _font = pNode->font();
   genProps->pFontPreviewLabel->setFont( _font );
   connect( genProps->pChooseFontBtn, SIGNAL( clicked() ), this, SLOT( slotChooseFont() ) );
   genProps->pPenColorBtn->setColor( pNode->lineColor() );
   genProps->pFillColorBtn->setColor( pNode->backColor());
   genProps->pLineThicknessSpinbox->setValue( pNode->lineWidth() );
}

void PropertiesDlg::setupConnProps()
{
	MyMapConnection* pTmpConn = pNode->parentConnection();
	connProps->pConnColorBtn->setColor( pTmpConn->linePen().color() );
  connProps->pConnLineStyleCombo->insertItem(i18n("No pen"), 0);
  connProps->pConnLineStyleCombo->insertItem("_________", 1);
  connProps->pConnLineStyleCombo->insertItem("- - - - -", 2);
  connProps->pConnLineStyleCombo->insertItem("* * * * *", 3);
  connProps->pConnLineStyleCombo->insertItem("-*-*-*-*-", 4);
  connProps->pConnLineStyleCombo->insertItem("-**-**-**", 5);

  switch( pTmpConn->lineStyle() )
  {
    case NoPen:  connProps->pConnLineStyleCombo->setCurrentItem(0); break;
    case SolidLine:  connProps->pConnLineStyleCombo->setCurrentItem(1); break;
    case DashLine:  connProps->pConnLineStyleCombo->setCurrentItem(2); break;
    case DotLine:  connProps->pConnLineStyleCombo->setCurrentItem(3); break;
    case DashDotLine:  connProps->pConnLineStyleCombo->setCurrentItem(4); break;
    case DashDotDotLine:  connProps->pConnLineStyleCombo->setCurrentItem(5); break;
  }

  connProps->pConnStyleCombo->insertItem(i18n("Simple Line"), 0);
  connProps->pConnStyleCombo->insertItem(i18n("Bezier Line"), 1);
  if ( pTmpConn->style() == mmcSIMPLELINE )
  	connProps->pConnStyleCombo->setCurrentItem(0);
  else
  	connProps->pConnStyleCombo->setCurrentItem(1);
  	
	connProps->pConnThicknessSpinbox->setValue( pTmpConn->linePen().width() );
//	connProps->pCopyLineStyleCheckbox->setChecked( pNode->isCopyConnStyle() );
//	slotChangeCopyFromEntry();
}

void PropertiesDlg::slotOKClicked()
{
  // General Properties
  pNode->setText( genProps->pTextEdit->text() );
	if ( genProps->pItemStyleCombo->currentText() == i18n("Ellipse") )  pNode->setStyle(mmELLIPSE);
	else
	if ( genProps->pItemStyleCombo->currentText() == i18n("Rectangle") )  pNode->setStyle(mmRECTANGLE);
	else
	if ( genProps->pItemStyleCombo->currentText() == i18n("Round Rectangle") )  pNode->setStyle(mmROUNDRECT);
	else
	if ( genProps->pItemStyleCombo->currentText() == i18n("Fork") )  pNode->setStyle(mmFORKSTYLE);
	else
	if ( genProps->pItemStyleCombo->currentText() == i18n("Transparent") )  pNode->setStyle(mmTRANSPARENT);

  pNode->setLineColor( genProps->pPenColorBtn->color());

  switch( genProps->pLineStyleCombo->currentItem() )
  {
    case 0:  pNode->setLineStyle( NoPen ); break;
    case 1:  pNode->setLineStyle( SolidLine ); break;
    case 2:  pNode->setLineStyle( DashLine ); break;
    case 3:  pNode->setLineStyle( DotLine ); break;
    case 4:  pNode->setLineStyle( DashDotLine ); break;
    case 5:  pNode->setLineStyle( DashDotDotLine ); break;
  }

  pNode->setTextColor( genProps->pTextColorBtn->color());
  pNode->setBackColor( genProps->pFillColorBtn->color());
  pNode->setLineWidth( genProps->pLineThicknessSpinbox->value() );
  pNode->setFont( _font );

  // Connector Properties
//  pNode->setCopyConnStyle( connProps->pCopyLineStyleCheckbox->isChecked() );
//  if ( pNode->isCopyConnStyle )
//    pNode->copyConnStyle();
  MyMapConnection* pTmpConn = pNode->parentConnection();
  if ( pTmpConn )
  {
  	QPen conpen( connProps->pConnColorBtn->color(), connProps->pConnThicknessSpinbox->value() );
  	pTmpConn->setLinePen( conpen );
  	switch( connProps->pConnLineStyleCombo->currentItem() )
  	{
  	  case 0:  pTmpConn->setLineStyle( NoPen ); break;
  	  case 1:  pTmpConn->setLineStyle( SolidLine ); break;
   		case 2:  pTmpConn->setLineStyle( DashLine ); break;
    	case 3:  pTmpConn->setLineStyle( DotLine ); break;
    	case 4:  pTmpConn->setLineStyle( DashDotLine ); break;
    	case 5:  pTmpConn->setLineStyle( DashDotDotLine ); break;
  	}
  	switch( connProps->pConnStyleCombo->currentItem() )
  	{
  		case 0:  pTmpConn->setStyle( mmcSIMPLELINE );
  		case 1:  pTmpConn->setStyle( mmcBEZIERLINE );
  	}
  }
}

void PropertiesDlg::slotChooseFont()
{
	KFontDialog::getFont( _font );
	genProps->pFontPreviewLabel->setFont( _font );
}

void PropertiesDlg::slotChangeCopyFromEntry()
{
  connProps->pConnColorBtn->setEnabled( !connProps->pCopyLineStyleCheckbox->isChecked() );
  connProps->pConnLineStyleCombo->setEnabled( !connProps->pCopyLineStyleCheckbox->isChecked() );
  connProps->pConnThicknessSpinbox->setEnabled( !connProps->pCopyLineStyleCheckbox->isChecked() );
}

void PropertiesDlg::showTab( int i )
{
	if ( i == 0 )
		showPage( genProps );
	else if ( i == 1 )
		showPage( connProps );
}


